@@ -25,10 +25,10 @@
 
 #include "gtest/gtest.h"
 #include "aom_dsp_rtcd.h"
-#include "EbDefinitions.h"
-#include "EbUtility.h"
-#include "EbRestoration.h"
-#include "EbUnitTestUtility.h"
+#include "definitions.h"
+#include "utility.h"
+#include "restoration.h"
+#include "unit_test_utility.h"
 #include "random.h"
 #include "util.h"
 
@@ -329,19 +329,31 @@ TEST_P(PixelProjErrorLbdTest, DISABLED_SpeedTest) {
     run_speed_test(256, 0, 0);
 }
 
-static const PixelProjErrorTestParam lbd_test_vector[] = {
-    make_tuple(svt_av1_lowbd_pixel_proj_error_sse4_1,
-               svt_av1_lowbd_pixel_proj_error_c),
-    make_tuple(svt_av1_lowbd_pixel_proj_error_avx2,
-               svt_av1_lowbd_pixel_proj_error_c),
-#if EN_AVX512_SUPPORT
-    make_tuple(svt_av1_lowbd_pixel_proj_error_avx512,
-               svt_av1_lowbd_pixel_proj_error_c)
-#endif
-};
+#if ARCH_X86_64
+INSTANTIATE_TEST_SUITE_P(
+    SSE4_1, PixelProjErrorLbdTest,
+    ::testing::Values(make_tuple(svt_av1_lowbd_pixel_proj_error_sse4_1,
+                                 svt_av1_lowbd_pixel_proj_error_c)));
 
-INSTANTIATE_TEST_CASE_P(RST, PixelProjErrorLbdTest,
-                        ::testing::ValuesIn(lbd_test_vector));
+INSTANTIATE_TEST_SUITE_P(
+    AVX2, PixelProjErrorLbdTest,
+    ::testing::Values(make_tuple(svt_av1_lowbd_pixel_proj_error_avx2,
+                                 svt_av1_lowbd_pixel_proj_error_c)));
+
+#if EN_AVX512_SUPPORT
+INSTANTIATE_TEST_SUITE_P(
+    AVX512, PixelProjErrorLbdTest,
+    ::testing::Values(make_tuple(svt_av1_lowbd_pixel_proj_error_avx512,
+                                 svt_av1_lowbd_pixel_proj_error_c)));
+#endif
+#endif  // ARCH_X86_64
+
+#if ARCH_AARCH64
+INSTANTIATE_TEST_SUITE_P(
+    NEON, PixelProjErrorLbdTest,
+    ::testing::Values(make_tuple(svt_av1_lowbd_pixel_proj_error_neon,
+                                 svt_av1_lowbd_pixel_proj_error_c)));
+#endif  // ARCH_AARCH64
 
 class PixelProjErrorHbdTest : public PixelProjErrorTest<uint16_t> {
   protected:
@@ -380,242 +392,195 @@ TEST_P(PixelProjErrorHbdTest, MatchTestWithExtremeValue) {
     run_extreme_test();
 }
 
-static const PixelProjErrorTestParam hbd_test_vector[] = {
-    make_tuple(svt_av1_highbd_pixel_proj_error_sse4_1,
-               svt_av1_highbd_pixel_proj_error_c),
-    make_tuple(svt_av1_highbd_pixel_proj_error_avx2,
-               svt_av1_highbd_pixel_proj_error_c)};
+#if ARCH_X86_64
+INSTANTIATE_TEST_SUITE_P(
+    SSE4_1, PixelProjErrorHbdTest,
+    ::testing::Values(make_tuple(svt_av1_highbd_pixel_proj_error_sse4_1,
+                                 svt_av1_highbd_pixel_proj_error_c)));
 
-INSTANTIATE_TEST_CASE_P(RST, PixelProjErrorHbdTest,
-                        ::testing::ValuesIn(hbd_test_vector));
+INSTANTIATE_TEST_SUITE_P(
+    AVX2, PixelProjErrorHbdTest,
+    ::testing::Values(make_tuple(svt_av1_highbd_pixel_proj_error_avx2,
+                                 svt_av1_highbd_pixel_proj_error_c)));
+#endif  // ARCH_X86_64
 
-// test svt_get_proj_subspace
-TEST(SelfGuidedToolsTest, GetProjSubspaceMatchTest) {
-    const int32_t pu_width = RESTORATION_PROC_UNIT_SIZE;
-    const int32_t pu_height = RESTORATION_PROC_UNIT_SIZE;
-    const int32_t width = 270, height = 256, stride = 300, out_stride = 300;
-    const int NUM_ITERS = 2000;
-    int i, j, k;
+#if ARCH_AARCH64
+INSTANTIATE_TEST_SUITE_P(
+    NEON, PixelProjErrorHbdTest,
+    ::testing::Values(make_tuple(svt_av1_highbd_pixel_proj_error_neon,
+                                 svt_av1_highbd_pixel_proj_error_c)));
+#endif  // ARCH_AARCH64
 
-    uint8_t *input_ = (uint8_t *)svt_aom_memalign(
-        32, stride * (height + 32) * sizeof(uint8_t));
-    uint8_t *output_ = (uint8_t *)svt_aom_memalign(
-        32, out_stride * (height + 32) * sizeof(uint8_t));
-    int32_t *tmpbuf = (int32_t *)svt_aom_memalign(32, RESTORATION_TMPBUF_SIZE);
-    uint8_t *input = input_ + stride * 16 + 16;
-    uint8_t *output = output_ + out_stride * 16 + 16;
-    int32_t *flt0 = tmpbuf;
-    int32_t *flt1 = flt0 + RESTORATION_UNITPELS_MAX;
-    int32_t flt_stride = ((width + 7) & ~7) + 8;
+typedef void (*GetProjSubspaceFunc)(const uint8_t *src8, int32_t width,
+                                    int32_t height, int32_t src_stride,
+                                    const uint8_t *dat8, int32_t dat_stride,
+                                    int32_t use_highbitdepth, int32_t *flt0,
+                                    int32_t flt0_stride, int32_t *flt1,
+                                    int32_t flt1_stride, int32_t *xq,
+                                    const SgrParamsType *params);
 
-    // check all the sg params
-    SVTRandom rnd(8, false);
-    for (int iter = 0; iter < NUM_ITERS; ++iter) {
-        if (iter == 0) {
-            // prepare src data and recon data
-            for (i = -16; i < height + 16; ++i) {
-                for (j = -16; j < width + 16; ++j) {
-                    input[i * stride + j] = rnd.random();
-                    if (iter == 0)
-                        output[i * stride + j] = input[i * stride + j];
-                    else
-                        output[i * stride + j] = rnd.random();
+template <typename Sample>
+class GetProjSubspaceTest
+    : public ::testing::TestWithParam<GetProjSubspaceFunc> {
+  public:
+    void SetUp() override {
+        test_impl_ = GetParam();
+        input_ = (Sample *)svt_aom_memalign(
+            32, stride * (height + 32) * sizeof(Sample));
+        output_ = (Sample *)svt_aom_memalign(
+            32, out_stride * (height + 32) * sizeof(Sample));
+        tmpbuf_ = (int32_t *)svt_aom_memalign(32, RESTORATION_TMPBUF_SIZE);
+    }
+
+    void TearDown() override {
+        svt_aom_free(input_);
+        svt_aom_free(output_);
+        svt_aom_free(tmpbuf_);
+    }
+
+    void run_test() {
+        const int32_t pu_width = RESTORATION_PROC_UNIT_SIZE;
+        const int32_t pu_height = RESTORATION_PROC_UNIT_SIZE;
+        const int NUM_ITERS = 2000;
+        int i, j, k;
+        Sample *input = input_ + stride * 16 + 16;
+        Sample *output = output_ + out_stride * 16 + 16;
+        int32_t width_sample[] = {128, 192, 256, 270};
+        for (int w_index = 0;
+             w_index < (int)(sizeof(width_sample) / sizeof(width_sample[0]));
+             w_index++) {
+            int32_t width = width_sample[w_index];
+            int32_t *flt0 = tmpbuf_;
+            int32_t *flt1 = flt0 + RESTORATION_UNITPELS_MAX;
+            int32_t flt_stride = ((width + 7) & ~7) + 8;
+
+            // check all the sg params
+            SVTRandom rnd(8, false);
+            for (int iter = 0; iter < NUM_ITERS; ++iter) {
+                // prepare src data and recon data
+                for (i = -16; i < height + 16; ++i) {
+                    for (j = -16; j < width + 16; ++j) {
+                        if (iter == 0)
+                            output[i * stride + j] = input[i * stride + j] =
+                                rnd.random();
+                        else if (iter == 1)
+                            output[i * stride + j] = input[i * stride + j] = 0;
+                        else {
+                            input[i * stride + j] = rnd.random();
+                            output[i * stride + j] = rnd.random();
+                        }
+                    }
+                }
+
+                for (int32_t ep = 0; ep < SGRPROJ_PARAMS; ++ep) {
+                    // apply selfguided filter to get A and b
+                    for (k = 0; k < height; k += pu_height) {
+                        for (j = 0; j < width; j += pu_width) {
+                            int32_t w = AOMMIN(pu_width, width - j);
+                            int32_t h = AOMMIN(pu_height, height - k);
+                            Sample *output_p = output + k * out_stride + j;
+                            int32_t *flt0_p = flt0 + k * flt_stride + j;
+                            int32_t *flt1_p = flt1 + k * flt_stride + j;
+                            assert(w * h <= RESTORATION_UNITPELS_MAX);
+
+                            svt_av1_selfguided_restoration_c(
+                                (uint8_t *)output_p,
+                                w,
+                                h,
+                                out_stride,
+                                flt0_p,
+                                flt1_p,
+                                flt_stride,
+                                ep,
+                                8,
+                                0);
+                        }
+                    }
+
+                    aom_clear_system_state();
+                    int32_t xqd_c[2] = {0};
+                    int32_t xqd_asm[2] = {0};
+                    const SgrParamsType *const params =
+                        &svt_aom_eb_sgr_params[ep];
+                    uint8_t *input_p = sizeof(*input) == sizeof(uint16_t)
+                                           ? (CONVERT_TO_BYTEPTR(input))
+                                           : (uint8_t *)input;
+                    uint8_t *output_p = sizeof(*output) == sizeof(uint16_t)
+                                            ? (CONVERT_TO_BYTEPTR(output))
+                                            : (uint8_t *)output;
+                    int32_t use_highbitdepth =
+                        sizeof(*input) == sizeof(uint16_t) ? 1 : 0;
+
+                    svt_get_proj_subspace_c(input_p,
+                                            width,
+                                            height,
+                                            stride,
+                                            output_p,
+                                            out_stride,
+                                            use_highbitdepth,
+                                            flt0,
+                                            flt_stride,
+                                            flt1,
+                                            flt_stride,
+                                            xqd_c,
+                                            params);
+                    test_impl_(input_p,
+                               width,
+                               height,
+                               stride,
+                               output_p,
+                               out_stride,
+                               use_highbitdepth,
+                               flt0,
+                               flt_stride,
+                               flt1,
+                               flt_stride,
+                               xqd_asm,
+                               params);
+                    ASSERT_EQ(xqd_c[0], xqd_asm[0])
+                        << "xqd_asm[0] does not match with xqd_asm[0] with "
+                           "iter "
+                        << iter << " ep " << ep;
+                    ASSERT_EQ(xqd_c[1], xqd_asm[1])
+                        << "xqd_asm[1] does not match with xqd_asm[1] with "
+                           "iter "
+                        << iter << " ep " << ep;
                 }
             }
-        } else {
-            // prepare src data and recon data
-            for (i = -16; i < height + 16; ++i) {
-                for (j = -16; j < width + 16; ++j) {
-                    input[i * stride + j] = 0;
-                    if (iter == 0)
-                        output[i * stride + j] = input[i * stride + j];
-                    else
-                        output[i * stride + j] = 0;
-                }
-            }
-        }
-
-        for (int32_t ep = 0; ep < SGRPROJ_PARAMS; ++ep) {
-            // apply selfguided filter to get A and b
-            for (k = 0; k < height; k += pu_height) {
-                for (j = 0; j < width; j += pu_width) {
-                    int32_t w = AOMMIN(pu_width, width - j);
-                    int32_t h = AOMMIN(pu_height, height - k);
-                    uint8_t *output_p = output + k * out_stride + j;
-                    int32_t *flt0_p = flt0 + k * flt_stride + j;
-                    int32_t *flt1_p = flt1 + k * flt_stride + j;
-                    assert(w * h <= RESTORATION_UNITPELS_MAX);
-
-                    svt_av1_selfguided_restoration_avx2(output_p,
-                                                        w,
-                                                        h,
-                                                        out_stride,
-                                                        flt0_p,
-                                                        flt1_p,
-                                                        flt_stride,
-                                                        ep,
-                                                        8,
-                                                        0);
-                }
-            }
-
-            aom_clear_system_state();
-            int32_t xqd_c[2] = {0};
-            int32_t xqd_asm[2] = {0};
-            const SgrParamsType *const params = &svt_aom_eb_sgr_params[ep];
-            svt_get_proj_subspace_c(input,
-                                    width,
-                                    height,
-                                    stride,
-                                    output,
-                                    out_stride,
-                                    0,
-                                    flt0,
-                                    flt_stride,
-                                    flt1,
-                                    flt_stride,
-                                    xqd_c,
-                                    params);
-            svt_get_proj_subspace_avx2(input,
-                                       width,
-                                       height,
-                                       stride,
-                                       output,
-                                       out_stride,
-                                       0,
-                                       flt0,
-                                       flt_stride,
-                                       flt1,
-                                       flt_stride,
-                                       xqd_asm,
-                                       params);
-            ASSERT_EQ(xqd_c[0], xqd_asm[0])
-                << "xqd_asm[0] does not match with xqd_asm[0] with iter "
-                << iter << " ep " << ep;
-            ASSERT_EQ(xqd_c[1], xqd_asm[1])
-                << "xqd_asm[1] does not match with xqd_asm[1] with iter "
-                << iter << " ep " << ep;
         }
     }
 
-    svt_aom_free(input_);
-    svt_aom_free(output_);
-    svt_aom_free(tmpbuf);
+  private:
+    static const int32_t height = 256, stride = 300, out_stride = 300;
+    GetProjSubspaceFunc test_impl_;
+    Sample *input_;
+    Sample *output_;
+    int32_t *tmpbuf_;
+};
+
+using GetProjSubspaceTestLbd = GetProjSubspaceTest<uint8_t>;
+using GetProjSubspaceTestHbd = GetProjSubspaceTest<uint16_t>;
+
+TEST_P(GetProjSubspaceTestLbd, MatchTest) {
+    run_test();
+}
+TEST_P(GetProjSubspaceTestHbd, MatchTest) {
+    run_test();
 }
 
-// test svt_get_proj_subspace
-TEST(SelfGuidedToolsTest, GetProjSubspaceMatchTestHbd) {
-    const int32_t pu_width = RESTORATION_PROC_UNIT_SIZE;
-    const int32_t pu_height = RESTORATION_PROC_UNIT_SIZE;
-    const int32_t width = 270, height = 256, stride = 300, out_stride = 300;
-    const int NUM_ITERS = 2000;
-    int i, j, k;
+#if ARCH_X86_64
+INSTANTIATE_TEST_SUITE_P(AVX2, GetProjSubspaceTestLbd,
+                         ::testing::Values(svt_get_proj_subspace_avx2));
 
-    uint16_t *input_ = (uint16_t *)svt_aom_memalign(
-        32, stride * (height + 32) * sizeof(uint16_t));
-    uint16_t *output_ = (uint16_t *)svt_aom_memalign(
-        32, out_stride * (height + 32) * sizeof(uint16_t));
-    int32_t *tmpbuf = (int32_t *)svt_aom_memalign(32, RESTORATION_TMPBUF_SIZE);
-    uint16_t *input = input_ + stride * 16 + 16;
-    uint16_t *output = output_ + out_stride * 16 + 16;
-    int32_t *flt0 = tmpbuf;
-    int32_t *flt1 = flt0 + RESTORATION_UNITPELS_MAX;
-    int32_t flt_stride = ((width + 7) & ~7) + 8;
+INSTANTIATE_TEST_SUITE_P(AVX2, GetProjSubspaceTestHbd,
+                         ::testing::Values(svt_get_proj_subspace_avx2));
+#endif  // ARCH_X86_64
 
-    // check all the sg params
-    SVTRandom rnd(8, false);
-    for (int iter = 0; iter < NUM_ITERS; ++iter) {
-        if (iter == 0) {
-            // prepare src data and recon data
-            for (i = -16; i < height + 16; ++i) {
-                for (j = -16; j < width + 16; ++j) {
-                    input[i * stride + j] = rnd.random();
-                    if (iter == 0)
-                        output[i * stride + j] = input[i * stride + j];
-                    else
-                        output[i * stride + j] = rnd.random();
-                }
-            }
-        } else {
-            // prepare src data and recon data
-            for (i = -16; i < height + 16; ++i) {
-                for (j = -16; j < width + 16; ++j) {
-                    input[i * stride + j] = 0;
-                    if (iter == 0)
-                        output[i * stride + j] = input[i * stride + j];
-                    else
-                        output[i * stride + j] = 0;
-                }
-            }
-        }
-
-        for (int32_t ep = 0; ep < SGRPROJ_PARAMS; ++ep) {
-            // apply selfguided filter to get A and b
-            for (k = 0; k < height; k += pu_height) {
-                for (j = 0; j < width; j += pu_width) {
-                    int32_t w = AOMMIN(pu_width, width - j);
-                    int32_t h = AOMMIN(pu_height, height - k);
-                    uint16_t *output_p = output + k * out_stride + j;
-                    int32_t *flt0_p = flt0 + k * flt_stride + j;
-                    int32_t *flt1_p = flt1 + k * flt_stride + j;
-                    assert(w * h <= RESTORATION_UNITPELS_MAX);
-
-                    svt_av1_selfguided_restoration_avx2((uint8_t *)output_p,
-                                                        w,
-                                                        h,
-                                                        out_stride,
-                                                        flt0_p,
-                                                        flt1_p,
-                                                        flt_stride,
-                                                        ep,
-                                                        8,
-                                                        0);
-                }
-            }
-
-            aom_clear_system_state();
-            int32_t xqd_c[2] = {0};
-            int32_t xqd_asm[2] = {0};
-            const SgrParamsType *const params = &svt_aom_eb_sgr_params[ep];
-            svt_get_proj_subspace_c(CONVERT_TO_BYTEPTR(input),
-                                    width,
-                                    height,
-                                    stride,
-                                    CONVERT_TO_BYTEPTR(output),
-                                    out_stride,
-                                    1,
-                                    flt0,
-                                    flt_stride,
-                                    flt1,
-                                    flt_stride,
-                                    xqd_c,
-                                    params);
-            svt_get_proj_subspace_avx2(CONVERT_TO_BYTEPTR(input),
-                                       width,
-                                       height,
-                                       stride,
-                                       CONVERT_TO_BYTEPTR(output),
-                                       out_stride,
-                                       1,
-                                       flt0,
-                                       flt_stride,
-                                       flt1,
-                                       flt_stride,
-                                       xqd_asm,
-                                       params);
-
-            ASSERT_EQ(xqd_c[0], xqd_asm[0])
-                << "xqd_asm[0] does not match with xqd_asm[0] with iter "
-                << iter << " ep " << ep;
-            ASSERT_EQ(xqd_c[1], xqd_asm[1])
-                << "xqd_asm[1] does not match with xqd_asm[1] with iter "
-                << iter << " ep " << ep;
-        }
-    }
-
-    svt_aom_free(input_);
-    svt_aom_free(output_);
-    svt_aom_free(tmpbuf);
-}
+#if ARCH_AARCH64
+INSTANTIATE_TEST_SUITE_P(NEON, GetProjSubspaceTestLbd,
+                         ::testing::Values(svt_get_proj_subspace_neon));
+INSTANTIATE_TEST_SUITE_P(NEON, GetProjSubspaceTestHbd,
+                         ::testing::Values(svt_get_proj_subspace_neon));
+#endif  // ARCH_AARCH64
 
 }  // namespace
